

#import python modules
import numpy as np
import time

#import compiled fortran library
import ex13lib

import sys
import os

# import Atomwrite.py
import Atomwrite
import Init

#whether or not to use the 3d visualization
UseVisual = False
if UseVisual:
    #import custom visualization library
    import atomvis


#====================================== GLOBAL VARIABLES ===============================================

"""
# target temperature
T = 1.0
# time step for molecular dynamics simulation
dt = 0.001
# total time for simulation
tTot = 0.5
# number of steps between velocity rescalings
RescaleFreq = 10
# Number of spatial dimensions
Dim = 3
# fractional energy tolerance for line search
EFracTolLS = 1.e-8
# fractional energy tolerance for conjugate gradient search
EFracTolCG = 1.e-10
# initial step amount for LS and CG
dx = 0.001
# number of integration steps for first equilibration in mean-square displacement calculations
# Recommended: 1.e4
NStepsEquil1 = 1.e3
# number of integration steps for second equilibration in mean-square displacement calculations
# Recommended: 1.e4
NStepsEquil2 = 1.e3
# number of integration steps from the production run in mean-square displacement calculations
# Recommended: 1.e5
NStepsProd = 1.e4
#set the random number seed; useful for debugging
#np.random.seed(342324)
"""

#everything below assumes unit atomic masses,
#such that forces = accelerations.




#========================================           =====================================================
#======================================== FUNCTIONS =====================================================
#========================================           =====================================================



#==================================== WRAPPER FUNCTIONS =================================================


def CalcEnergyForces(Pos, Vel, Forces, EParams, T, dt):
	"""Wrapper for energy and force calculation.
Input:
    Pos: (N,3) position array
    Forces: (N,3) force array
Output:
    PEnergy: potential energy, float
    Forces: (N,3) force array (updated)
"""
	Np = EParams[0]
	Ns = EParams[1]
	L = EParams[2]
	Cut = EParams[3]
	EpsPP = EParams[4]
	EpsSS = EParams[5]
	EpsSP = EParams[6]
	SigPP = EParams[7]
	SigSS = EParams[8]
	SigSP = EParams[9]
	Dim = EParams[10]
	
	return ex13lib.calcenergyforces(Pos, Vel, Np, Ns, L, Cut, EpsPP, EpsSS, EpsSP, SigPP, SigSS, SigSP, T, dt, Forces, Dim)






#=======================================================================================
#             Molecular dyanmics functions
#=======================================================================================




def RescaleVelocities(Vel, T, Np):
	"""Rescales velocities in the system to the target temperature. Keeping velocities of S-atoms at zero.
Input:
    Vel: (N,3) array of atomic velocities
    T: target temperature
Output:
    Vel: same as above
"""
	#Pick out just the P-atom velocities
	VelP = Vel[0:Np]
	#recenter to zero net momentum (assuming all masses same)
	VelpP = VelP - VelP.mean(axis=0)
	#find the total kinetic energy
	KE = 0.5 * np.sum(VelP * VelP)
	#find velocity scale factor from ratios of kinetic energy
	VScale = np.sqrt(1.5 * len(VelP) * T / KE)
	VelP = VelP * VScale
	#replace the P-atom velocities with the rescaled values
	Vel[0:Np] = VelP
	return Vel  

#===============================================================

def InitVelocities(N, T):
    """Returns an initial random velocity set.
Input:
    N: number of atoms
    T: target temperature
Output:
    Vel: (N,3) array of atomic velocities
"""
    Vel = np.random.rand(N, 3)
    Vel = RescaleVelocities(Vel, T, N)
    return Vel

#===============================================================



def PeriodicBound(Pos, Np, Ns, L):
	"""Moves S-atoms back into the box.
Input:
	Pos: (Np+Ns,3) array of positions
	Np: number of P atoms
	Ns: number of N atoms
Output:
	Pos: (Np+Ns, 3) array of positions """
	for i in range(Np,Np+Ns):
		xi = Pos[i,0]
		yi = Pos[i,1]
		zi = Pos[i,2]
		
		if xi**2 > 0.25 * L**2 and xi > 0.:
			Pos[i,0] = Pos[i,0] - L
		if xi**2 > 0.25 * L**2 and xi < 0.:
			Pos[i,0] = Pos[i,0] + L
		if yi**2 > 0.25 * L**2 and yi > 0.:
			Pos[i,1] = Pos[i,1] - L
		if yi**2 > 0.25 * L**2 and yi < 0.:
			Pos[i,1] = Pos[i,1] + L
		if zi**2 > 0.25 * L**2 and zi > 0.:
			Pos[i,2] = Pos[i,1] - L
		if zi**2 > 0.25 * L**2 and zi < 0.:
			Pos[i,2] = Pos[i,1] + L

	return Pos

#===============================================================



def CountSP(Pos, Np, Ns, CountCut):
	"""Counts the number of S "bound" to the polymer, i.e. within distance CountCut of a P atom
Input:
	Pos: (Np+Ns,3) array of positions
	Np: number of P atoms
	Ns: number of N atoms
	CountCut: max distance for S-atom to be considered "bound" to P-atom
Output:
	BoundSP: number of S-atoms "bound" to polymer """
	#loop over S-atoms
	BoundSP = 0
	CountCut2 = CountCut**2
	for i in range (Np, Np+Ns):
		#loop over P-atoms
		for j in range (Np):
			rij = Pos[i] - Pos[j]
			d2 = np.dot(rij, rij)
			if d2 < CountCut2:
				BoundSP += 1
				break
	return BoundSP
	
#===============================================================
def CountSS(Pos, Np, Ns, CountCut):
	for i in range (Np, Np+Ns):
		for j in range (Np, Np+Ns):
			if j == 1: continue
			

#=================================================================


def CalcEndToEnd(Pos, Np):
	R = Pos[0,:] - Pos[Np-1,:]
	D2 = np.dot(R,R)
	D = np.sqrt(D2)
	return D


#==================================================================
"""
def Andersen(Vel, N, T):
	i = np.random.randint(0, N)
	n = np.random.normal(
	"""





def InstTemp(Vel):
    """Returns the instantaneous temperature.
Input:
    Vel: (N,3) array of atomic velocities
Output:
    Tinst: float
"""
    return np.sum(Vel * Vel) / (3. * len(Vel))





def VVIntegrate(Pos, Vel, Accel, Np, Ns, L, Cut, dt, EpsPP, EpsSS, EpsSP, SigPP, SigSS, SigSP, Dim, T):
	"""Wrapper for the Velocity Verlet
Input:
	Pos: (N,3) position array
	Vel: (N,3) velocity array
	Accel: (N,3) accleration array
Output:
	Pos: (N,3) position array
	Vel: (N,3) velocity array
	Accel: (N,3) accleration array
	PEnergy: potential energy, float
	KEnergy: kinetic energy, float
"""
	return ex13lib.vvintegrate(Pos, Vel, Accel, Np, Ns, L, Cut, dt, T, EpsPP, EpsSS, EpsSP, SigPP, SigSS, SigSP, Dim)






def VVerlet(Pos, Vel, Accel, tTot, dt, RescaleFreq, CountSPFreq, CountCut, WritePdbFreq, EParams, T):
	"""Runs the velocity verlet alorithm with velocity rescaling to fix temperature.
Input:
	Pos: (N,3) array of atomic positions
	Vel: (N,3) array of atomic velocities
    Accel: (N,3) array of acceleration vectors
Output:
	Pos: (N,3) array of atomic positions
	Vel: (N,3) array of atomic velocities
"""
	Np = EParams[0]
	Ns = EParams[1]
	L = EParams[2]
	Cut = EParams[3]
	EpsPP = EParams[4]
	EpsSS = EParams[5]
	EpsSP = EParams[6]
	SigPP = EParams[7]
	SigSS = EParams[8]
	SigSP = EParams[9]
	Dim = EParams[10]
	
	AtomNamesP = ["C"]
	AtomNamesS = ["N"]
	
	Filename = "AA.pdb"
	Atomwrite.WritePdb(Filename, Pos, Np, Ns, AtomNamesP, AtomNamesS, First = True)
	
	t = 0
	i = 0
	EndToEnd = []
	while t < tTot:
		t = t + dt
		i = i+1
		(Pos, Vel, Accel, KEnergy, PEnergy) = VVIntegrate(Pos, Vel, Accel, Np, Ns, L, Cut, dt, EpsPP, EpsSS, EpsSP, SigPP, SigSS, SigSP, Dim, T)
		temp = InstTemp(Vel)
		if i % WritePdbFreq == 0:
			Energy = KEnergy + PEnergy
			D = CalcEndToEnd(Pos, Np)
			print "%-8.6f" % t + "\t" + "%-8.6f" % Energy + "\t" + "%-8.6f" % D# + "\t" + "%-8.6f" % BoundSP
			Atomwrite.WritePdb(Filename, Pos, Np, Ns, AtomNamesP, AtomNamesS, First = False)
	return Pos, Vel